/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookwyrm.
 *
 * The Initial Developer of the Original Code is
 * M. Damian Dollahite.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   M.D. Dollahite (master.ryukage@gmail.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ::::: Constants ::::: */
const NS_STORYLIST         = "http://purl.org/int-fiction/metadata/1.1/";
const NS_USERDATA          = "http://purl.org/int-fiction/bookwyrm/rdf/";
const NS_CATEGORY          = "bookwyrm:cat#";
const NS_DC                = "http://purl.org/dc/elements/1.1/";
const URI_TITLE_PRED       = NS_DC+"title";
const URI_DESCRIPTION_PRED = NS_DC+"description";
const STORY_LIST_FILE      = "bookwyrm.rdf";
const APP_ID               = "{c2c1f16b-b212-47e8-a9e5-edc6d0337f60}";
const APP_VER              = "0.1";


/* ::::: Globals ::::: */

var gStrings1;   // Manager strings
var gStrings2;   // Storylist strings

// get a reference to the prompt service component.
var nsiPromptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                    getService(Components.interfaces.nsIPromptService);


// windowOnLoad sets this as a reference to the storyview element
var gStorylist = null;


/* ::::: Dialog Boxes ::::: */

// Add a story to the database
function addNewStory()
{
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"].
                createInstance(nsIFilePicker);
    fp.init(window, gStrings1.getString("addstory.filepicker.title"), nsIFilePicker.modeOpen);
    
    var terps = gStorylist.rdfds.getNode("bookwyrm:platforms").getChildren();
    while(terps.hasMoreElements())
    {
        var terp = terps.getNext();
        fp.appendFilter(
            terp.getTarget(NS_USERDATA + "filter").getValue(),
            terp.getTarget(NS_USERDATA + "extensions").getValue());
    }

    fp.appendFilters(nsIFilePicker.filterAll);

    var rv = fp.show();
    if (rv == nsIFilePicker.returnOK)
    {
        gStorylist.addStory(fp.file, fp.file.leafName);
    }
}

// Load a website in the browser
function launchWebsite(aBranch)
{
    try
    {
        var pref = Components.classes["@mozilla.org/preferences;1"].getService(Components.interfaces.nsIPref);
        var uri = pref.CopyUnicharPref("bookwyrm.web."+aBranch+".uri");

    	window.open(uri, "Web"+aBranch+"Window", "menubar,location,toolbar,resizable,status,scrollbars");
    }
    catch (e)
    {
        // No default pref or couldn't create pref service
    }
}

// Show the about box
function openAboutDialog()
{
    openDialog("aboutDialog.xul", "AboutBookwyrm", "modal,centerscreen,chrome,resizable=no");
}

function openPreferences()
{
    openDialog("chrome://bookwyrm/content/pref/pref.xul","BookwyrmPrefWindow", 
        "chrome,titlebar,resizable,modal");
}

/* ::::: Topic Observers ::::: */

var categoryChangeObserver =
{
    observe: function (storylist, topic, catURI)
    {
        var RDFService = Components.classes["@mozilla.org/rdf/rdf-service;1"].
            getService().QueryInterface(Components.interfaces.nsIRDFService);

        var cat = RDFService.GetResource(catURI);
        var usr = RDFService.GetResource(NS_USERDATA+"user");
        var cmd = document.getElementById("cmd_DeleteCategory");

        if(cmd)
        {
            if(storylist.database.hasArcOut(cat, usr))
                cmd.removeAttribute("disabled");
            else
                cmd.setAttribute("disabled", "true");
        }
    }
};

/* ::::: Event Handlers ::::: */

// Fixup View menu
function onViewPopupShowing(aEvent)
{
    var popup = aEvent.target;
    var menuitem = popup.firstChild;

    while(menuitem != null)
    {
        if(menuitem.hasAttribute("toolbarid"))
        {
            var toolbar = document.getElementById(menuitem.getAttribute("toolbarid"));
            menuitem.setAttribute("checked", toolbar.getAttribute("collapsed") == "true" ? "false" : "true");
        }

        menuitem = menuitem.nextSibling;
    }
    
    // Fix the infobar item as well, needs special handling
    menuitem = document.getElementById("view-menu-infobar");
    menuitem.setAttribute("checked", gStorylist.hideinfobar ? "false" : "true");
}

// Fixup Toolbar menus
function onViewToolbarsPopupShowing(aEvent)
{
    var popup = aEvent.target;
    var i;

    // Empty the menu
    for (i = popup.childNodes.length-1; i >= 0; --i) 
    {
        var deadItem = popup.childNodes[i];
        if (deadItem.hasAttribute("toolbarindex"))
            popup.removeChild(deadItem);
    }

    var firstMenuItem = popup.firstChild;

    var toolbox = document.getElementById("manager-toolbox");
    //var toolbar = toolbox.firstChild;

    for (i = 0; i < toolbox.childNodes.length; ++i)
    //while(toolbar != null)
    {
        var toolbar = toolbox.childNodes[i];
        var toolbarName = toolbar.getAttribute("toolbarname");
        var type = toolbar.getAttribute("type");

        if (toolbarName && type != "menubar")
        {
            var menuItem = document.createElement("menuitem");

            menuItem.setAttribute("toolbarindex", i);
            menuItem.setAttribute("type", "checkbox");
            menuItem.setAttribute("label", toolbarName);
            menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
            menuItem.setAttribute("checked", toolbar.getAttribute("collapsed") != "true");
            popup.insertBefore(menuItem, firstMenuItem);

            menuItem.addEventListener("command", onViewToolbarCommand, false);
        }

        //toolbar = toolbar.nextSibling;
    }
    
    // Add our special toolbars that aren't in the main toolbox
    var toolbar = document.getElementById("manager-searchbar");
    var toolbarName = toolbar.getAttribute("toolbarname");
    var type = toolbar.getAttribute("type");

    if (toolbarName && type != "menubar")
    {
        var menuItem = document.createElement("menuitem");

        menuItem.setAttribute("toolbarindex", -1);
        menuItem.setAttribute("type", "checkbox");
        menuItem.setAttribute("label", toolbarName);
        menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
        menuItem.setAttribute("checked", toolbar.getAttribute("collapsed") != "true");
        popup.insertBefore(menuItem, firstMenuItem);

        menuItem.addEventListener("command", onViewToolbarCommand, false);
    }
    
    toolbar = document.getElementById("manager-commandbar");
    toolbarName = toolbar.getAttribute("toolbarname");
    type = toolbar.getAttribute("type");

    if (toolbarName && type != "menubar")
    {
        var menuItem = document.createElement("menuitem");

        menuItem.setAttribute("toolbarindex", -2);
        menuItem.setAttribute("type", "checkbox");
        menuItem.setAttribute("label", toolbarName);
        menuItem.setAttribute("accesskey", toolbar.getAttribute("accesskey"));
        menuItem.setAttribute("checked", toolbar.getAttribute("collapsed") != "true");
        popup.insertBefore(menuItem, firstMenuItem);

        menuItem.addEventListener("command", onViewToolbarCommand, false);
    }
}

// Handle toolbar toggle
function onViewToolbarCommand(aEvent)
{
    var toolbox = document.getElementById("manager-toolbox");
    var index = aEvent.originalTarget.getAttribute("toolbarindex");
    if(index == -1)
        var toolbar = document.getElementById("manager-searchbar");
    else if(index == -2)
        var toolbar = document.getElementById("manager-commandbar");
    else
        var toolbar = toolbox.childNodes[index];

    toolbar.collapsed = aEvent.originalTarget.getAttribute("checked") != "true";
    document.persist(toolbar.id, "collapsed");
}


/* ::::: Customization ::::: */

// Call the toolbar customization dialog.
function ManagerCustomizeToolbar()
{
    // Disable the toolbar context menu items
    var menubar = document.getElementById("manager-menubar");
    for (var i = 0; i < menubar.childNodes.length; ++i)
        menubar.childNodes[i].setAttribute("disabled", true);

    var cmd = document.getElementById("cmd_CustomizeToolbars");
    cmd.setAttribute("disabled", "true");

    window.openDialog("chrome://global/content/customizeToolbar.xul", "CustomizeToolbar",
                      "chrome,all,dependent", document.getElementById("manager-toolbox"));
}

function ManagerToolboxCustomizeDone(aToolboxChanged)
{
    // Update global UI elements that may have been added or removed
    if (aToolboxChanged)
    {
        updateSidebarButton();
    }

    // Re-enable parts of the UI we disabled during the dialog
    var menubar = document.getElementById("manager-menubar");
    for (var i = 0; i < menubar.childNodes.length; ++i)
        menubar.childNodes[i].setAttribute("disabled", false);
    var cmd = document.getElementById("cmd_CustomizeToolbars");
    cmd.removeAttribute("disabled");

    // XXX Shouldn't have to do this, but I do
    window.focus();
}

// Handle enabling and disabling of story commands
function enableStoryCommands()
{
    var disable = gStorylist.selected == null;
    var storyCommands = document.getElementById("cmdset-story").childNodes;

    for(var i = 0; i < storyCommands.length; i++)
    {
        // Global overlay has a goSetCommandEnabled() function,
        // but it's more efficient to do it ourself since we
        // already have a reference to the DOM node
        if(disable)
            storyCommands[i].setAttribute("disabled", "true");
        else
            storyCommands[i].removeAttribute("disabled");
    }
}

function toggleSidebar(forceOn)
{
    var sidebar = document.getElementById("manager-sidebar");
    var splitter = document.getElementById("sidebar-splitter");

    if(sidebar.hasAttribute("collapsed") || forceOn)
    {
        sidebar.removeAttribute("collapsed");
        splitter.removeAttribute("disabled");
    }
    else
    {
        sidebar.setAttribute("collapsed", "true");
        splitter.setAttribute("disabled", "true");
    }
    
    updateSidebarButton();
}

function updateSidebarButton()
{
    var sidebarButton = document.getElementById("sidebar-button");

    if(sidebarButton)
    {
        var sidebar = document.getElementById("manager-sidebar");
        if(sidebar && sidebar.getAttribute("collapsed") != "true")
            sidebarButton.setAttribute("checked", "true");
        else
            sidebarButton.removeAttribute("checked");
    }
}

function toggleInfobar()
{
    if(gStorylist.getAttribute("hideinfobar") == "true")
        gStorylist.removeAttribute("hideinfobar");
    else
        gStorylist.setAttribute("hideinfobar", "true");
    document.persist(gStorylist.id, "hideinfobar");
}

function toggleStatusbar()
{
    var statusbar = document.getElementById("manager-statusbar");
	if(statusbar)
    {
        if(statusbar.getAttribute("collapsed") == "true")
            statusbar.removeAttribute("collapsed");
        else
            statusbar.setAttribute("collapsed", "true");
        document.persist(statusbar.id, "collapsed");
    }
}

/* ::::: Window Operations ::::: */

// Startup Procedure
function windowOnLoad()
{
    window.tryToClose = windowOnClose;

    // Register Observers
    var observerService =
        Components.classes["@mozilla.org/observer-service;1"].
            getService(Components.interfaces.nsIObserverService);
    observerService.addObserver(categoryChangeObserver, "storyview:change-category", false);
    observerService.addObserver(categoryChangeObserver, "storyview:refresh", false);

    // Initialize the database
    gStorylist = document.getElementById("manager-listview");
    initDatabase();

    // Add customizeDone handler
    var toolbox = document.getElementById("manager-toolbox");
    toolbox.customizeDone = ManagerToolboxCustomizeDone;

    // Grab some global handles to our string bundles
    gStrings1 = document.getElementById("manager-strings");
    gStrings2 = gStorylist.mStrings;

    // Initialize UI elements
    setDefaultStatus();
    updateSidebarButton();
}

// Initialize the database - can also reset the database after preferences change
function initDatabase()
{
    // Finalize any open database
    gStorylist.closeDatabase();

    // Get the database file from preferences
    var file = null;
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
        getService(Components.interfaces.nsIPrefService).getBranch("bookwyrm.database.");
    try
    {
        file = prefs.getComplexValue("file", Components.interfaces.nsILocalFile);
    }
    catch(e)
    {
        // Preference not set -- default to bookwyrm.rdf in profile directory
        file = Components.classes["@mozilla.org/file/directory_service;1"].
    	   getService(Components.interfaces.nsIProperties).
    	       get("ProfD", Components.interfaces.nsILocalFile);
    	file.append(STORY_LIST_FILE);  // const STORY_LIST_FILE = "bookwyrm.rdf"
        prefs.setComplexValue("file", Components.interfaces.nsILocalFile, file);
    }

    // Convert local path to URL
    var ios = Components.classes["@mozilla.org/network/io-service;1"].
    	getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file").
    	QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var rdfURL = fileHandler.getURLSpecFromFile(file);

    // If the file doesn't exist or is empty, copy our default template
    if(!file.exists() || file.fileSize == 0)
    {
        var template = Components.classes["@mozilla.org/file/directory_service;1"].
    	   getService(Components.interfaces.nsIProperties).
    	       get("ProfD", Components.interfaces.nsILocalFile);
    	template.append("extensions");
        template.append(APP_ID);
        template.append("defaults");
    	template.append(STORY_LIST_FILE);  // const STORY_LIST_FILE = "bookwyrm.rdf"
    	template.copyTo(file.parent, file.leafName);
    	
    	gStorylist.initBlankDatabase(rdfURL, fileHandler.getURLSpecFromFile(template));
    }

    // Open the database in the storyview
    gStorylist.openDatabase(rdfURL);
}

// Control the statusbar
function setStatus(aStatus)
{
    var panel = document.getElementById("status-mainpanel");
    panel.label = aStatus;
}

function setDefaultStatus()
{
    var panel = document.getElementById("status-mainpanel");
    panel.label = gStrings1.getString("status.default");
}

// Control busy mode
function setBusyMode(aStatus, aProgressListener)
{
    setStatus(aStatus);
    document.documentElement.setAttribute("wait-cursor", "true");
    
    var throbber = document.getElementById("navigator-throbber");
    throbber.setAttribute("busy", "true");

    var progresspanel = document.getElementById("status-progresspanel");
    var progressmeter = progresspanel.firstChild;
    
    if(aProgressListener != null)
    {
        progressmeter.mode = "normal";
        progressmeter.value = 0;

        progresspanel.updateInterval = window.setInterval(
            function() { progressmeter.value = aProgressListener.percent; }, 100);
    }
    else
    {
        progressmeter.mode = "undetermined";
    }

    progresspanel.removeAttribute("collapsed");
}

function endBusyMode()
{
    setDefaultStatus();
    document.documentElement.removeAttribute("wait-cursor");

    var throbber = document.getElementById("navigator-throbber");
    throbber.removeAttribute("busy");

    var progresspanel = document.getElementById("status-progresspanel");

    window.clearInterval(progresspanel.updateInterval);
    progresspanel.updateInterval = null;
    
    progresspanel.setAttribute("collapsed", "true");
    progresspanel.firstChild.value = 0;
}

// Shutdown Procedure
function windowOnUnload()
{
    gStorylist.closeDatabase();
    gStorylist = null;

    // Unregister Observers
    var observerService =
        Components.classes["@mozilla.org/observer-service;1"].
            getService(Components.interfaces.nsIObserverService);
    observerService.removeObserver(categoryChangeObserver, "storyview:change-category");
    observerService.removeObserver(categoryChangeObserver, "storyview:refresh");
}

// Finalize Window
function windowOnClose()
{
    // delegate to standard function in globalOverlay.js
    return closeWindow(false);
}


/* ::::: Service Functions ::::: */

// Strip the namespace from an RDF predicate
function stripNamespace(pred)
{
    var last_slash = pred.lastIndexOf("/");
    var last_hash = pred.lastIndexOf("#");
    var ns_end = (last_slash < last_hash) ? last_hash : last_slash;
    return pred.substring(ns_end+1, pred.length);
}

// generic functions for toggling the state of an element
function toggleElement(id){ toggleAttribute(id, "hidden"); }
function toggleCheck(id) { toggleAttribute(id, "checked"); }
function toggleAttribute(id, attr)
{
    var elem = document.getElementById(id);
	if(elem)
    {
        if(elem.getAttribute(attr) == "true")
            elem.removeAttribute(attr);
        else
            elem.setAttribute(attr, "true");
    }
}

// Force an element on
function showElement(id)
{
    var elem = document.getElementById(id);
    if(elem) elem.removeAttribute("hidden");
}

// Force an element off
function hideElement(id)
{
    var elem = document.getElementById(id);
    if(elem) elem.setAttribute("hidden", "true");
}

function openExternal(aPath)
{
    var uri = Components.classes["@mozilla.org/network/standard-url;1"].
        createInstance(Components.interfaces.nsIURI);
    uri.spec = "file:///" + aPath;

    var protocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"].
            getService(Components.interfaces.nsIExternalProtocolService);
    protocolSvc.loadUrl(uri);

    return;
}

function getLocalFileFromNativePathOrUrl(aPathOrUrl)
{
    if (aPathOrUrl.substring(0,7) == "file://") 
    {
        // if this is a URL, get the file from that
        ioSvc = Components.classes["@mozilla.org/network/io-service;1"].
            getService(Components.interfaces.nsIIOService);

        // XXX it's possible that using a null char-set here is bad
        const fileUrl = ioSvc.newURI(aPathOrUrl, null, null).
            QueryInterface(Components.interfaces.nsIFileURL);
        return fileUrl.file.clone().
            QueryInterface(Components.interfaces.nsILocalFile);
    }
    else
    {
        // if it's a pathname, create the nsILocalFile directly
        f = Components.classes["@mozilla.org/file/local;1"].
            createInstance(Components.interfaces.nsILocalFile);
        f.initWithPath(aPathOrUrl);
        
        return f;
    }
}

