/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookwyrm.
 *
 * The Initial Developer of the Original Code is
 * M. Damian Dollahite.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   M.D. Dollahite (master.ryukage@gmail.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const fields = ["extensions", "filter", "external"];

var listview = null;
var rdfds = null;

var deleted = [];

function onOK()
{
    try
    {
        var terp = listview.firstChild;
        var seq = rdfds.getNode("bookwyrm:platforms");

        while(terp != null)
        {
            var res = rdfds.getNode(terp.id);
            seq.addChild(res, false);
            res.addTargetOnce("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", 
                                    rdfds.getNode("http://purl.org/int-fiction/metadata/type/Platform"));
            
            res.addTargetOnce("http://purl.org/dc/elements/1.1/title", terp.getAttribute("title"));
            res.addTargetOnce("http://purl.org/dc/elements/1.1/description", terp.getAttribute("description"));

            for(var i = 0; i < fields.length; i++)
            {
                var field = fields[i];
                res.addTargetOnce(
                    "http://purl.org/int-fiction/bookwyrm/rdf/"+field,
                    terp.getAttribute(field));
            }

            res.addTargetOnce(
                "http://purl.org/int-fiction/bookwyrm/rdf/fileprop",
                rdfds.getNode(terp.getAttribute("fileprop")));

            terp = terp.nextSibling;
        }

        for(var i = 0; i < deleted.length; i++)
        {
            rdfds.deleteNode(deleted[i]);
        }
        
        rdfds.save();
    }
    catch(e)
    {
        // never loaded |interpreters| pane
    }
}

function Startup()
{
    parent.hPrefWindow.registerOKCallbackFunc(onOK);

    // Get the database file from preferences
    var file = null;
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
        getService(Components.interfaces.nsIPrefService).getBranch("bookwyrm.database.");
    try
    {
        file = prefs.getComplexValue("file", Components.interfaces.nsILocalFile);
    }
    catch(e)
    {
        // TODO: What to do if the database hasn't been set yet?
    }

    // Convert local path to URL
    var ios = Components.classes["@mozilla.org/network/io-service;1"].
    	getService(Components.interfaces.nsIIOService);
    var fileHandler = ios.getProtocolHandler("file").
    	QueryInterface(Components.interfaces.nsIFileProtocolHandler);
    var rdfURL = fileHandler.getURLSpecFromFile(file);
    
    rdfds = new RDFDataSource(rdfURL);
    listview = document.getElementById("interpreters-view");
    refreshListview();
}

function refreshListview()
{
    // Empty any old entries from the list
    while(listview.hasChildNodes())
        listview.removeChild(listview.firstChild);

    // Retrieve the current set
    var seq = rdfds.getNode("bookwyrm:platforms").getChildren();
    
    while(seq.hasMoreElements())
    {
        var terp = seq.getNext();
        var newterp = document.createElement("platform");
        newterp.id = terp.getValue();

        newterp.setAttribute("title", terp.getTarget("http://purl.org/dc/elements/1.1/title").getValue());
        newterp.setAttribute("description", terp.getTarget("http://purl.org/dc/elements/1.1/description").getValue());

        for(var i = 0; i < fields.length; i++)
        {
            var field = fields[i];
            newterp.setAttribute(field,
                terp.getTarget("http://purl.org/int-fiction/bookwyrm/rdf/"+field).getValue());
        }
        
        // Need to do this specially because it's a resource instead of a literal
        newterp.setAttribute("fileprop", terp.getTarget("http://purl.org/int-fiction/bookwyrm/rdf/fileprop").getValue());
        
        if(terp.getTarget("http://purl.org/int-fiction/bookwyrm/rdf/external").getValue() == "true")
        {
            // Convert filename to nsILocalFile
            var f = Components.classes["@mozilla.org/file/local;1"].
                createInstance(Components.interfaces.nsILocalFile);
            f.initWithPath(terp.getValue());
            
            // Convert local path to URL
            var ios = Components.classes["@mozilla.org/network/io-service;1"].
            	getService(Components.interfaces.nsIIOService);
            var fileHandler = ios.getProtocolHandler("file").
            	QueryInterface(Components.interfaces.nsIFileProtocolHandler);
            var url = fileHandler.getURLSpecFromFile(f);

            newterp.setAttribute("image", "moz-icon:"+url+"?size=32");
        }
        else
        {
            newterp.setAttribute("image", "chrome://bookwyrm/skin/icon.png");
        }

        listview.appendChild(newterp);
    }
}

function deleteInterpreter(elt)
{
    try
    {
        deleted.push(elt.id);  // Remember that we deleted this, but don't remove from RDF until dialogAccept
        elt.parentNode.removeChild(elt);
    }
    catch(e)
    {
        // never loaded |interpreters| pane or invalid elt
    }
}

function addInterpreter()
{
    var strings = document.getElementById("interpretersBundle");
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                .createInstance(nsIFilePicker);
    fp.init(window, strings.getString("addterp.filepicker.title"), nsIFilePicker.modeOpen);
    fp.appendFilters(nsIFilePicker.filterApps);
    fp.appendFilters(nsIFilePicker.filterAll);

    var rv = fp.show();
    if (rv == nsIFilePicker.returnOK)
    {
        var newterp = document.createElement("platform");
        newterp.id = fp.file.path;
        newterp.setAttribute("title", fp.file.leafName);
        newterp.setAttribute("external", "true");
        newterp.setAttribute("open", "true");
        listview.appendChild(newterp);
    }
}


