/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookwyrm.
 *
 * The Initial Developer of the Original Code is
 * M. Damian Dollahite.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   M.D. Dollahite (master.ryukage@gmail.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// Story node wrapper object
//  This substitutes for a story element from story.xml, 
//  since there isn't one available yet when we're adding
//  a brand new entry to the database.
function Story(aNode)
{
    var rdfNode = aNode;
    var fieldNS = "http://purl.org/int-fiction/metadata/1.1/";

    this.getField = function(fieldname)
    {
        var val = rdfNode.getTarget(fieldNS + fieldname);
        return val ? val.getValue() : "";
    };
    this.setField = function(fieldname, value)
        { rdfNode.addTargetOnce(fieldNS + fieldname, value); };

    this.getTarget = function(predicate)
    {
        var val = rdfNode.getTarget(predicate);
        return val ? val.getValue() : "";
    };
    this.setTarget = function(predicate, value)
        { rdfNode.addTargetOnce(predicate, value); };
        
    this.ref = rdfNode.getValue();
}

// Globals
var gStorylist = null;
var gStory = null;
var gStrings = null;

// Initialize Properties Dialog
function initDialog()
{
    gStorylist = window.arguments[0];
    gStory = new Story(window.arguments[1]);
    gStrings = document.getElementById("strings");

    // Set up the platform menu
    var platformMenu = document.getElementById("platform-menu-popup");
    var platformList = gStorylist.getNode("bookwyrm:platforms").getChildren();

    while(platformList.hasMoreElements())
    {
        var terp = platformList.getNext();
        var menuItem = document.createElement("menuitem");
        
        menuItem.setAttribute("value", terp.getValue());
        menuItem.setAttribute("label", terp.getTarget("http://purl.org/dc/elements/1.1/description").getValue());
        
        platformMenu.appendChild(menuItem);
    }

    // We'll generate the category menu below, 
    // so that we can set up the current selection at the same time.

    // Initialize controls
    if(gStory != null)
    {
        // icon
        document.getElementById("story-icon").src = gStory.getField("icon");

        // interpreter
        var platformMenulist = document.getElementById("platform-input");
        var terp = gStory.getField(platformMenulist.getAttribute("storyfield"));
        for(var i = 0; i < platformMenu.childNodes.length; i++)
        {
            if(platformMenu.childNodes[i].getAttribute("value") == terp)
            {
                platformMenulist.selectedIndex = i;
                break;
            }
        }

        // categories
        var catMenu = document.getElementById("categories-input");
        var catList = gStorylist.getNode("bookwyrm:categories").getChildren();
        
        while(catList.hasMoreElements())
        {
            var cat = catList.getNext();
            var usercat = cat.getTarget("http://purl.org/int-fiction/bookwyrm/rdf/user");

            if(usercat && usercat.getValue() == "true")
            {
                var listItem = document.createElement("checkbox");  //("listitem");

                listItem.setAttribute("value", cat.getValue());
                listItem.setAttribute("label", cat.getTarget("http://purl.org/dc/elements/1.1/title").getValue());

                if(cat.getChildIndex(window.arguments[1]) != -1)
                    listItem.setAttribute("checked", "true"); //("selected", "true");

                catMenu.appendChild(listItem);
            }
        }

        // theme
        var themeGroup = document.getElementById("theme-input");
        var themeVal = gStory.getField("presentationProfile");
        var radio = themeGroup.firstChild;

        while(radio != null)
        {
            if(radio.value == themeVal)
            {
                themeGroup.selectedItem = radio;
                break;
            }
            
            radio = radio.nextSibling;
        }
        
        // zarfian
        var zarfianList = document.getElementById("zarfian-input");
        var zarfianVal = gStory.getField("zarfian");
        var zitem = zarfianList.firstChild.firstChild;

        while(zitem != null)
        {
            if(zitem.value == zarfianVal)
            {
                zarfianList.selectedItem = zitem;
                break;
            }
            
            zitem = zitem.nextSibling;
        }

        // textboxes can be batched
        var textboxes = document.getElementsByTagName("textbox");
        for(var i = 0; i < textboxes.length; i++)
        {
            if(textboxes[i].hasAttribute("storyfield"))
            {
                textboxes[i].value = gStory.getField(
                    textboxes[i].getAttribute("storyfield"));
            }
        }

        // mainfile
        var mainfileInput = document.getElementById("file-input");
        mainfileInput.value = gStory.getField(
            mainfileInput.getAttribute("storyfield"));

        // For the time being, we cannot allow changing the file,
        // so disable the textbox and hide the button
        //var filebox = document.getElementById("file-input");
        //filebox.value = gStory.ref;
        //filebox.readonly = true;
        //filebox.nextSibling.hidden = true;
        
        var titlebox = document.getElementById("title-input");
        titlebox.focus();
    }
}

// Apply changes made
function applyChanges()
{
    // Update values
    if(gStory != null)
    {
        // interpreter
        var platformMenulist = document.getElementById("platform-input");

        if(platformMenulist.value == "auto")
        {
            var defaultPlatform = getDefaultPlatform(gStory.ref);
            if(!defaultPlatform)
            {
                // Error Message
                Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
                    getService(Components.interfaces.nsIPromptService).
                        alert(window,
                            gStrings.getString("alert.cannotSelectDefaultPlatform.caption"),
                            gStrings.getString("alert.cannotSelectDefaultPlatform.text"));

                // Don't allow dialog closure
                return false;
            }

            gStory.setField(
                platformMenulist.getAttribute("storyfield"),
                defaultPlatform);
        }
        else
        {
            gStory.setField(
                platformMenulist.getAttribute("storyfield"),
                gStorylist.getNode(platformMenulist.value));
        }

        // icon
        gStory.setField("icon", gStorylist.getNode(document.getElementById("story-icon").src));

        // categories
        var catMenu = document.getElementById("categories-input");
        var listItem = catMenu.firstChild;

        while(listItem != null)
        {
            var cat = gStorylist.getNode(listItem.getAttribute("value"));

            if(listItem.getAttribute("checked") == "true") //("selected") == "true"
            {
                if(cat.getChildIndex(window.arguments[1]) == -1)
                    cat.addChild(window.arguments[1]);
            }
            else
            {
                cat.removeChild(window.arguments[1]);
            }

            listItem = listItem.nextSibling;
        }

        // theme
        var themeGroup = document.getElementById("theme-input");
        gStory.setField("presentationProfile", themeGroup.value);

        // zarfian
        var zarfianList = document.getElementById("zarfian-input");
        gStory.setField("zarfian", zarfianList.value);

        // textboxes can be batched
        var textboxes = document.getElementsByTagName("textbox");
        for(var i = 0; i < textboxes.length; i++)
        {
            if(textboxes[i].hasAttribute("storyfield"))
            {
                dump("\nSetting field: "+textboxes[i].getAttribute("storyfield")+
                    ":="+textboxes[i].value);
                gStory.setField(
                    textboxes[i].getAttribute("storyfield"),
                    textboxes[i].value);
            }
        }

        // mainfile
        var mainfileInput = document.getElementById("file-input");
        gStory.setField(
            mainfileInput.getAttribute("storyfield"),
            gStorylist.getNode(mainfileInput.value));
    }

    return true;
}

function categoriesSelectAll()
{
    var cats = document.getElementById("categories-input").childNodes;

    for(var i = 0; i < cats.length; i++)
        cats[i].setAttribute("checked", "true");
}

function categoriesSelectNone()
{
    var cats = document.getElementById("categories-input").childNodes;
    
    for(var i = 0; i < cats.length; i++)
        cats[i].removeAttribute("checked");
}

function selectNewIcon(aFpTitle)
{
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                .createInstance(nsIFilePicker);
    fp.init(window, aFpTitle, nsIFilePicker.modeOpen);
    fp.appendFilters(nsIFilePicker.filterImages);

    var rv = fp.show();
    if (rv == nsIFilePicker.returnOK)
    {
        document.getElementById("story-icon").src = fp.fileURL.spec;
    }
}

function getDefaultPlatform(aImagename)
{
    dump("\nPlatform Detect: imagename:="+aImagename);
    // Extract extension from the filename
    var ext = "*" + aImagename.substr(aImagename.lastIndexOf("."), aImagename.length);
    ext = ext.toLowerCase();

    // Go through the platform list and look for a match
    var platformList = gStorylist.getNode("bookwyrm:platforms").getChildren();

    while(platformList.hasMoreElements())
    {
        var terp = platformList.getNext();
        var exts = terp.getTarget("http://purl.org/int-fiction/bookwyrm/rdf/extensions").getValue();
        exts = exts.toLowerCase();

        if(exts.indexOf(ext) >= 0)
            return terp;
    }

    return null;
}

/*function browseForMainfile(aFpTitle)
{
    const nsIFilePicker = Components.interfaces.nsIFilePicker;
    var fp = Components.classes["@mozilla.org/filepicker;1"]
                .createInstance(nsIFilePicker);
    fp.init(window, aFpTitle, nsIFilePicker.modeOpen);
    fp.appendFilters(nsIFilePicker.filterAll);

    var rv = fp.show();
    if (rv == nsIFilePicker.returnOK)
    {
        return fp.file.path;
    }

    return null;
}*/

function windowClosing()
{
    gStorylist = null;
    gStory = null;
    return true;
}
