/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookwyrm.
 *
 * The Initial Developer of the Original Code is
 * M. Damian Dollahite.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   M.D. Dollahite (master.ryukage@gmail.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the NPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ::::: Constants ::::: */

const COMMON_STORY_FIELDS =
[
    "title",
    "byline",
//  "authorEmail",
    "desc",
//    "icon",

    "separator",

    "genre",
    "version",
    "releaseDate",
    "length",
    "zarfian",

//    "separator",

//    "platform",
//    "lastPlayed",
//    "mainfile"
];

function StoryviewObserver(header)
{
    var storyview = null;

    if(header.storyview)
         storyview = header.ownerDocument.getElementById(header.storyview);

    this.observe = function(target, topic, data)
    {
        if(storyview && target != storyview) return;

        switch(topic)
        {
        case "storyview:open-database":
            header.categoryButton.firstChild.datasources = storyview.datasources;
        case "storyview:change-category":
            header.categoryButton.label = storyview.categoryName;
            break;
        case "storyview:change-sorting":
            header.quicksortButton.label = header.mStrings.getString("quicksort.button.prefix") +
                " " + header.mStrings.getString("field." + storyview.sortField + ".label");
            break;
        case "storyview:change-sorting-direction":
            header.quickdirectionButton.setAttribute("class",
                "quickdirection-button sort-" + storyview.sortDirection);
            break;
        case "storyview:refresh":
            header.categoryButton.firstChild.datasources = storyview.datasources;
            header.categoryButton.label = storyview.categoryName;
            header.quicksortButton.label = header.mStrings.getString("quicksort.button.prefix") +
                " " + header.mStrings.getString("field." + storyview.sortField + ".label");
            header.quickdirectionButton.setAttribute("class",
                "quickdirection-button sort-" + storyview.sortDirection);
            break;
        }
    }
}

/* ::::: Event Handlers ::::: */

/* 
 *   TODO: All the functions below should be altered to locate the storyview through
 *     the storyviewheader element instead of using gStoryview, which is only meant
 *     for use in "manager.js". The problem is getting a reference to the header...
 *
 *   There are a few oncommand handlers in "story.xml#listheader" that need this also
 */

// Handle user category change
function onChangeCategory(aEvent)
{
    gStorylist.category = aEvent.originalTarget.getAttribute("value");
}

function onChangeCategoryNS(aEvent)
{
    gStorylist.ref = aEvent.originalTarget.getAttribute("id");
}

// Add some extra stuff to the change category menu
function onCategoryPopupShowing(aEvent)
{
    var popup = aEvent.target;
    var strings = gStorylist.mStrings;
    var addItem = document.createElement("menuitem");
    addItem.setAttribute("label", strings.getString("category.add.label"));

    // Clear the stale list
    for(var i = popup.childNodes.length-1; i >= 1 ; i--)
        popup.removeChild(popup.childNodes[i]);

    // Rebuild the menu
    popup.builder.rebuild();
    popup.appendChild(document.createElement("menuseparator"));
    popup.appendChild(addItem);
    addItem.addEventListener("command", onAddCategoryCommand, false);
    
    // Add a separator between built-in and user categories
    for(var i = 1; i < popup.childNodes.length; i++)
    {
        if(popup.childNodes[i].getAttribute("usercat") == "true")
        {
            popup.insertBefore(
                document.createElement("menuseparator"),
                popup.childNodes[i]);
            break;
        }
    }
}

function onAddCategoryCommand(aEvent)
{
    gStorylist.addCategory();
    aEvent.preventBubble();
}

// Configure the sort quickmenu
function onQuicksortPopupShowing(aEvent)
{
    var popup = aEvent.target;
    var sortby = gStorylist.sortField;
    var strings = gStorylist.mStrings;
    var i;

    // Empty the menu
    for (i = popup.childNodes.length-1; i >= 0; --i)
    {
        var deadItem = popup.childNodes[i];
        if (deadItem.hasAttribute("value"))
            popup.removeChild(deadItem);
    }

    var firstMenuItem = popup.firstChild;

    for (i = 0; i < COMMON_STORY_FIELDS.length; ++i)
    {
        var fieldname = COMMON_STORY_FIELDS[i];
        if(fieldname == "separator")
        {
            var sep = document.createElement("menuseparator");
            sep.setAttribute("value", "------");  // dummy value so it'll be removed when emptying the menu
            popup.insertBefore(sep, firstMenuItem);
            continue;
        }
        var menuItem = document.createElement("menuitem");

        menuItem.setAttribute("value", fieldname);
        menuItem.setAttribute("name", "quicksort");
        menuItem.setAttribute("type", "radio");
        menuItem.setAttribute("label", strings.getString("field." + fieldname + ".label"));
        menuItem.setAttribute("checked", sortby == fieldname);
        popup.insertBefore(menuItem, firstMenuItem);

        menuItem.addEventListener("command", onQuicksortCommand, false);
    }
}

function onQuicksortCommand(aEvent)
{
    gStorylist.sortField = aEvent.target.value;
    aEvent.preventBubble();
}

// Configure the fields quickmenu
function onQuickfieldsPopupShowing(aEvent)
{
    var popup = aEvent.target;
    var fields = gStorylist.storyfields;
    var strings = gStorylist.mStrings;
    var i;

    // Empty the menu
    for (i = popup.childNodes.length-1; i >= 0; --i)
    {
        var deadItem = popup.childNodes[i];
        if (deadItem.hasAttribute("value"))
            popup.removeChild(deadItem);
    }

    var firstMenuItem = popup.firstChild;

    for (i = 0; i < COMMON_STORY_FIELDS.length; ++i)
    {
        var fieldname = COMMON_STORY_FIELDS[i];
        if(fieldname == "separator")
        {
            var sep = document.createElement("menuseparator");
            sep.setAttribute("value", "------");  // dummy value so it'll be removed when emptying the menu
            popup.insertBefore(sep, firstMenuItem);
            continue;
        }
        var menuItem = document.createElement("menuitem");

        menuItem.setAttribute("value", fieldname);
        menuItem.setAttribute("type", "checkbox");
        menuItem.setAttribute("label", strings.getString("field." + fieldname + ".label"));
        menuItem.setAttribute("checked", fields.indexOf(fieldname) >= 0);
        if(fieldname == "title") menuItem.setAttribute("disabled", "true");
        popup.insertBefore(menuItem, firstMenuItem);

        menuItem.addEventListener("command", onQuickfieldCommand, false);
    }
}

function onQuickfieldCommand(aEvent)
{
    var item = aEvent.target;
    gStorylist.showField(item.value, item.getAttribute("checked") == "true");
    aEvent.preventBubble();
}

